import wm from './lib/windowManager';
import { wait } from './lib/utils';
import { app } from 'electron';
import displayCrashScreen from './lib/crash';
import express from 'express';
import path from 'path';
import { createServer } from 'http';

declare global {
  namespace NodeJS {
    interface Process {
      nfenv?: any;
    }
  }
}

process.nfenv = {
  serverPort: 65034 as number, // todo: pick a random one instead, not for now bcuz easier to debug and use /eval endpoint
  crash: function (reason?: string) {
    console.error('[nfcrash] Simulating a crash with reason:', reason || '*no reason*');
    displayCrashScreen();
  }
}

async function startWebRoot() {
  const app = express();
  const server = createServer(app);

  app.use(express.static(path.join(__dirname, 'webroot')));

  app.get('/eval', (req: express.Request, res: express.Response): any => {
    const code = req.headers['x-code'] as string;
  
    if (!code) return res.status(400).send('No code provided');
  
    try {
      const result = (function(require) {
        return eval(code);
      })(require);
      res.status(200).send(result as string);
    } catch (error: any) {
      res.status(500).send(error.message as string);
    }
  });

  // to-do: add echo webhook endpoint and endpoints to create more webhook endpoints (and also delete those)

  server.listen(process.nfenv.serverPort, 'localhost');
};

startWebRoot().then(() => {
  app.on('ready', async () => {
    try {
      const mw = wm.create({
        w: 20,
        h: 20,
        whp: true,
        onCreate: (win: any) => {
          win.loadURL(`http://localhost:${process.nfenv.serverPort}/screens/mainmenu/test.html`);
        },
      });
  
      wm.move({ id: mw, x: 50, y: 50, p: true, fromCenter: true })
      await wait(500);
      wm.resize({ id: mw, w: 40, h: 40, smooth: true, p: true});
  
    } catch (e) {
      console.error(e);
      process.nfenv.crash();
    }
  });
})